//Globale Variablen
unsigned char buffer[512];
unsigned char *pbuffer=&buffer[0];


//Allgemeine Definitionen:
#define device			0x00	//0x00 fr Device 0, 0x10 fr Device 1


//IDE-Adressleitungs-Makros	(High-Nibble: CS0, CS1; Low-Nibble: DA0-DA2)
#define alternate_status	0x16
#define device_control		0x16
#define data			0x20
#define error			0x21
#define features		0x21
#define sector_count		0x22
#define lba0_7			0x23
#define lba8_15			0x24
#define lba16_23		0x25
#define lba24_27		0x26
#define status			0x27
#define command			0x27

                                    
//IDE-Komando-Makros
#define identify_device		0xEC
#define read			0x20




unsigned char ide_status(void)
{
  return(ide_read_byte(status));
}
 

unsigned char ide_error(void)
{
  return(ide_read_byte(error));
}


unsigned char ide_read_byte(unsigned char adresse)
{
  unsigned char temp;
  DDRA=0x00;
  DDRC=0x00;
  PORTA=0xFF;	
  PORTC=0xFF;   //PORTS auf Eingang schalten, Pullups on
  
  PORTD=adresse;	//Adresse an Datenleitung legen
  
  PORTE.2=0;	//Read-Bedingung
  delay_ms(1);	//Setup-Time
  
  temp=PINA;	//Daten lesen
  
  PORTE.2=1;  
  
  PORTD=0x00;             
  
  return(temp); 
}


unsigned int ide_read_word(unsigned char adresse)
{
  unsigned int temp;
  DDRA=0x00;
  DDRC=0x00;
  PORTA=0xFF;	
  PORTC=0xFF;   //PORTS auf Eingang schalten, Pullups on
  
  PORTD=adresse;	//Adresse an Datenleitung legen
  
  PORTE.2=0;	//Read-Bedingung
  delay_ms(1);	//Setup-Time
  
  temp=PINC;	//Daten lesen
  temp=temp<<8;
  temp=temp|PINA;
  
  PORTE.2=1;  

  PORTD=0x00;
  
  return(temp); 
}


unsigned char ide_read_sector(unsigned long int sector,unsigned char *pbuffer)
{ 
  unsigned long int temp;

  while((ide_status()&0x80)>=1)	//Auf bereitschaft warten
  {
  	//evt. Timeoff einbauen
  }
  
  ide_write_byte(lba24_27,device);	//Zugriff auf Device festlegen
                                                                      	
  while(((ide_status()&0x80)>=1)|(ide_status()&0x40)==0)	//Auf bereitschaft warten
  {
  	//evt. Timeoff
  }
  
  temp=sector&0x0F000000;	//Bytes 24-27
  temp=temp>>24;
  temp=temp|device|0xE0;	//LBA & Device-Bit dazunehmen
  ide_write_byte(lba24_27,temp);
  
  temp=sector&0x00FF0000;	//Bytes 16-23
  temp=temp>>16;
  ide_write_byte(lba16_23,temp);
  
  temp=sector&0x0000FF00;	//Bytes 8-15
  temp=temp>>8;
  ide_write_byte(lba8_15,temp);
  
  temp=sector&0x000000FF;	//Bytes 0-7
  ide_write_byte(lba0_7,temp);  
  
  ide_write_byte(sector_count,0x01);
  
  ide_write_byte(command,read);	//Befehl fr 'read-Device' schreiben
  
  ide_read_PIO(pbuffer);
  
  return(ide_error());
}




unsigned char ide_identify_device(void)
{
  while((ide_status()&0x80)>=1)	//Auf bereitschaft warten
  {
  	//evt. Timeoff einbauen
  }
  
  ide_write_byte(lba24_27,device);	//Zugriff auf Device festlegen
                                                                      	
  while(((ide_status()&0x80)>=1)|(ide_status()&0x40)==0)	//Auf bereitschaft warten
  {
  	//evt. Timeoff
  }
  
  ide_write_byte(command,identify_device);	//Befehl fr Identify-Device schreiben
  
  ide_read_PIO(pbuffer);
  
  
  
  return(ide_error());
}
          


void ide_write_byte(unsigned char adresse,unsigned char byte)
{
  DDRA=0xFF;   		//PORTA auf Ausgang stellen
  
  PORTD=adresse;	//Adressen an Port legen
  
  PORTA=byte;		//Daten an Port legen
  
  PORTE.1=0;		//Write-Befehl
  delay_ms(1);		//Setup-Time
  PORTE.1=1;		
  
  PORTD=0x00;
  
  DDRA=0x00;
  PORTA=0xFF;
}
  

void ide_read_PIO(unsigned char *pbuffer)
{
  unsigned int temp;
  unsigned int temp2;
  unsigned int t;

  for (t=0;t<=510;t+=2)
  {  
    while(((ide_status()&0x80)>=1)|((ide_status()&0x40)==0))
    {
    	//evt. Timeoff einbauen
    }
  

  
    temp=ide_read_word(data);	//Daten holen

    temp2=temp&0x00FF;
    *pbuffer=temp2;
  
    pbuffer++;
  
    temp2=temp&0xFF00;
    temp2=temp2>>8;
    *pbuffer=temp2;
  
    pbuffer++;
  }
}





void ide_init(void)
{  
  DDRA=0x00;            //PORTA auf Eingang schalten (D0-D7)
  PORTA=0xFF;		//PORTA Pullups einschalten
  
  DDRC=0x00;		//PORTC auf Eingang schalten (D8-D15)
  PORTC=0xFF;		//PORTC Pullups einschalten
  
  DDRD=0xFF;		//PORTD auf Ausgang schalten (AD0-AD2, CS0,CS1)
  PORTD=0x00;		
  
  DDRE=0x07;		//PORTE auf Ausgang schalten (Reset, IOW, IOR)
  PORTE=0x07;
  
  send_string("\n\rWarte auf Platte:");
  while(((ide_status()&0x80)>=1)|((ide_status()&0x40)==0))
  {
  	//evt. Timeoff setzen
  }                            
  send_string(" ok");   
  
  send_string("\n\rIdentify Device:"); 
  ide_identify_device();
  send_string(" ok");
  
  send_string("\n\rCylinder: ");
  send_hex(buffer[3]);
  send_hex(buffer[2]);
  send_string("\n\rHead: ");
  send_hex(buffer[7]);
  send_hex(buffer[6]);
  send_string("\n\rSectors: ");
  send_hex(buffer[13]);
  send_hex(buffer[12]);
  
  send_string("\n\rLBA-Support: "); 
  
  if (buffer[99]&0x01)
  {
    send_string("yes");
  }
  else
  {
    send_string("no! - abbort!");
    for (;;)
    {}
  } 
  
  send_string("\n\radressable Sectors (LBA): ");
  send_hex(buffer[121]);
  send_hex(buffer[120]);
  send_hex(buffer[123]);
  send_hex(buffer[122]);
  
}